/********************************** (C) COPYRIGHT *******************************
 * File Name          : ch32v20x_it.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main Interrupt Service Routines.
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/
#include "ch32v20x_it.h"
#include "board.h"
#include <rtthread.h>
void NMI_Handler(void) __attribute__((interrupt()));
void HardFault_Handler(void) __attribute__((interrupt()));
void EXTI2_IRQHandler(void) __attribute__((interrupt()));
void EXTI3_IRQHandler(void) __attribute__((interrupt()));

#define FREQ_SYS                96000000
#define MDELAYUS_LOOP_TICKS     4
#define __nop()                 __asm__ volatile("nop")
/*********************************************************************
 * @fn      mDelayuS
 *
 * @brief   uS ʱ
 *
 * @param   t       - ʱ
 *
 * @return  none
 */
void mDelayuS(uint16_t t)
{
    uint32_t i;

    i = t * (FREQ_SYS / 1000000 / MDELAYUS_LOOP_TICKS);

    do
    {
        __nop();
    } while(--i);
}
/*********************************************************************
 * @fn      NMI_Handler
 *
 * @brief   This function handles NMI exception.
 *
 * @return  none
 */
void NMI_Handler(void)
{
    GET_INT_SP();
    rt_interrupt_enter();
    rt_kprintf(" NMI Handler\r\n");
    rt_interrupt_leave();
    FREE_INT_SP();
}

/*********************************************************************
 * @fn      HardFault_Handler
 *
 * @brief   This function handles Hard Fault exception.
 *
 * @return  none
 */
void HardFault_Handler(void)
{
    GET_INT_SP();
    rt_interrupt_enter();
    rt_kprintf(" hardfult\r\n");
    printf("mepc :%08x\r\n", __get_MEPC());

    printf("mcause:%08x\r\n", __get_MCAUSE());

    printf("mtval :%08x\r\n", __get_MTVAL());

    while(1);
    rt_interrupt_leave();
    FREE_INT_SP();

}

void EXTI2_IRQHandler(void)
{
  if(EXTI_GetITStatus(EXTI_Line2)==SET)  //EXTI_GetITStatusȡжϱ־λ״̬EXTI߲ж򷵻SET򷵻RESET
  {
    int i;
    printf("Run at EXTI2\r\n");
    for(i=0;i<100;i++)
    {
        printf("%d\n",i);
        mDelayuS(10000);

    }
    EXTI_ClearITPendingBit(EXTI_Line2);  //жϱ־λ
  }
}

void EXTI3_IRQHandler(void)
{

  int i;
  if(EXTI_GetITStatus(EXTI_Line3)==SET)  //EXTI_GetITStatusȡжϱ־λ״̬EXTI߲ж򷵻SET򷵻RESET
  {
    printf("Run at EXTI3\r\n");
    for(i=100;i<500;i++)
    {
        printf("%d\n",i);
        mDelayuS(10000);
    }
    EXTI_ClearITPendingBit(EXTI_Line3);  //жϱ־λ
  }

}
